
library(s2dv)
library(multiApply)

ResidualCorr <- function(exp, ref, obs, time_dim = 'year', member_dim = NULL, alpha = 0.05, ncores = 1){
  
  ## Checkings
  if (!is.array(exp)){stop('"exp" must be an array.')}
  if (!is.array(obs)){stop('"obs" must be an array.')}
  if (!is.array(ref)){stop('"ref" must be an array.')}
  if (!identical(as.numeric(dim(exp)[time_dim]), as.numeric(dim(ref)[time_dim]))){stop('"time_dim" must have the same length in "exp" and "ref".')}
  if (!identical(as.numeric(dim(exp)[time_dim]), as.numeric(dim(obs)[time_dim]))){stop('"time_dim" must have the same length in "exp" and "obs".')}
  if (!is.numeric(alpha) | alpha <= 0 | alpha >= 1){stop('"alpha" must be a number between 0 and 1.')}
  if (!is.character(time_dim)){stop('"time_dim" must be a string.')}
  if (!is.null(member_dim)){
    if (!is.character(member_dim)){stop('"member_dim" must be either NULL or a string.')}
  }
  if (!is.null(ncores)){
    if (!is.numeric(ncores) | ncores < 1){stop('"ncores" must be either NULL or a positive integer.')}
  }
  
  ## Ensemble mean
  if (!is.null(member_dim)){
    exp <- multiApply::Apply(data = exp, target_dims = member_dim, fun = mean, na.rm = FALSE, ncores = ncores)$output1
    ref <- multiApply::Apply(data = ref, target_dims = member_dim, fun = mean, na.rm = FALSE, ncores = ncores)$output1
  }
  
  ## Residual correlation and its significance
  output <- multiApply::Apply(data = list(exp = exp, ref = ref, obs = obs),
                              target_dims = time_dim, fun = .ResidualCorr,
                              time_dim = time_dim, alpha = alpha, ncores = ncores)
  return(output)
}

.ResidualCorr <- function(exp, ref, obs, time_dim, alpha){
  
  if (!anyNA(exp) & !anyNA(ref) & !anyNA(obs)){
    
    ## Residuals of 'exp' and 'obs' (regressing 'ref' out in both 'exp' and 'obs')
    exp_res <- lm(formula = y ~ x, data = list(y = exp, x = ref), na.action = NULL)$residuals
    obs_res <- lm(formula = y ~ x, data = list(y = obs, x = ref), na.action = NULL)$residuals
    
    ## Residual correlation (and significance)
    residual_cor <- NULL
    residual_cor$r <- cor(exp_res,obs_res)
    
    ## Significance
    residual_cor$n_eff <- s2dv::Eno(data = obs_res, time_dim = time_dim, na.action = na.pass, ncores = 1)
    t_alpha2_n2 <- qt(p=alpha/2, df = residual_cor$n_eff-2, lower.tail = FALSE)
    t <- abs(residual_cor$r) * sqrt(residual_cor$n_eff-2) / sqrt(1-residual_cor$r^2)
    if (anyNA(c(t,t_alpha2_n2)) == FALSE & t >= t_alpha2_n2){
      residual_cor$sign <- TRUE
    } else {
      residual_cor$sign <- FALSE
    }
    
  } else {
    residual_cor <- list(r = NA, n_eff = NA, sign = NA)
  }
  
  return(residual_cor)
}
